use std;
use std::fmt::{self, Display};
use serde::{de, ser};
use crate::bipack_source::BipackError;

pub type Result<T> = std::result::Result<T, Error>;

#[derive(Debug)]
pub enum Error {
    Message(String),
    BadFormat(String),
    Eof,
    ExtraBytes,
    BadEncoding,
    NotSupported,
    NotImplemented,
    NotPossible
}

impl Display for Error {
    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
        match self {
            Error::Message(msg) => formatter.write_str(msg),
            _ => write!(formatter, "BiPack error: {:?}", self),
        }
    }
}

impl std::error::Error for Error {}

impl ser::Error for Error {
    fn custom<T: Display>(msg: T) -> Self {
        Error::Message(msg.to_string())
    }
}

impl de::Error for Error {
    fn custom<T: Display>(msg: T) -> Self {
        Error::Message(msg.to_string())
    }
}

impl From<BipackError> for Error {
    fn from(value: BipackError) -> Self {
        match &value {
            BipackError::NoDataError => Error::Eof,
            BipackError::BadEncoding(_) => Error::BadEncoding,
            // other => Error::Message(format!("BiPack error: {}", value))
        }
    }
}
